#[derive(serde::Serialize, serde::Deserialize, Clone, Copy, Debug)]
pub struct DomainXMLUuid(pub uuid::Uuid);

impl DomainXMLUuid {
    pub fn parse_from_str(s: &str) -> anyhow::Result<Self> {
        Ok(Self(uuid::Uuid::parse_str(s)?))
    }

    pub fn as_string(&self) -> String {
        self.0.to_string()
    }

    pub fn is_valid(&self) -> bool {
        log::debug!("UUID version ({}): {}", self.0, self.0.get_version_num());
        self.0.get_version_num() == 4
    }
}

/// OS information
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "os")]
pub struct OSXML {
    #[serde(rename(serialize = "@firmware"), default)]
    pub firmware: String,
    pub r#type: OSTypeXML,
    pub loader: Option<OSLoaderXML>,
}

/// OS Type information
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "os")]
pub struct OSTypeXML {
    #[serde(rename(serialize = "@arch"))]
    pub arch: String,
    #[serde(rename = "$value")]
    pub body: String,
}

/// OS Loader information
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "loader")]
pub struct OSLoaderXML {
    #[serde(rename(serialize = "@secure"))]
    pub secure: String,
}

/// Hypervisor features
#[derive(serde::Serialize, serde::Deserialize, Default)]
#[serde(rename = "features")]
pub struct FeaturesXML {
    pub acpi: ACPIXML,
}

/// ACPI feature
#[derive(serde::Serialize, serde::Deserialize, Default)]
#[serde(rename = "acpi")]
pub struct ACPIXML {}

/// Devices information
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "devices")]
pub struct DevicesXML {
    /// Graphics (used for VNC
    #[serde(skip_serializing_if = "Option::is_none")]
    pub graphics: Option<GraphicsXML>,

    /// Disks (used for storage)
    #[serde(default, rename = "disk")]
    pub disks: Option<DiskXML>, // TODO : change to vec
}

/// Screen information
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "graphics")]
pub struct GraphicsXML {
    #[serde(rename(serialize = "@type"))]
    pub r#type: String,
    #[serde(rename(serialize = "@socket"))]
    pub socket: String,
}

/// Disk information
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "disk")]
pub struct DiskXML {
    #[serde(rename(serialize = "@type"))]
    pub r#type: String,
    #[serde(rename(serialize = "@device"))]
    pub r#device: String,

    pub driver: DiskDriverXML,
    pub source: DiskSourceXML,
    pub target: DiskTargetXML,
    pub readonly: DiskReadOnlyXML,
    pub boot: DiskBootXML,
    pub address: DiskAddressXML,
}

#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "driver")]
pub struct DiskDriverXML {
    #[serde(rename(serialize = "@name"))]
    pub name: String,
    #[serde(rename(serialize = "@type"))]
    pub r#type: String,
}

#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "source")]
pub struct DiskSourceXML {
    #[serde(rename(serialize = "@file"))]
    pub file: String,
}

#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "target")]
pub struct DiskTargetXML {
    #[serde(rename(serialize = "@dev"))]
    pub dev: String,
    #[serde(rename(serialize = "@bus"))]
    pub bus: String,
}

#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "readonly")]
pub struct DiskReadOnlyXML {}

#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "boot")]
pub struct DiskBootXML {
    #[serde(rename(serialize = "@order"))]
    pub order: String,
}

#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "address")]
pub struct DiskAddressXML {
    #[serde(rename(serialize = "@type"))]
    pub r#type: String,
    #[serde(rename(serialize = "@controller"))]
    pub r#controller: String,
    #[serde(rename(serialize = "@bus"))]
    pub r#bus: String,
    #[serde(rename(serialize = "@target"))]
    pub r#target: String,
    #[serde(rename(serialize = "@unit"))]
    pub r#unit: String,
}

/// Domain RAM information
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "memory")]
pub struct DomainMemoryXML {
    #[serde(rename(serialize = "@unit"))]
    pub unit: String,

    #[serde(rename = "$value")]
    pub memory: usize,
}

/// Domain information, see https://libvirt.org/formatdomain.html
#[derive(serde::Serialize, serde::Deserialize)]
#[serde(rename = "domain")]
pub struct DomainXML {
    /// Domain type (kvm)
    #[serde(rename(serialize = "@type"))]
    pub r#type: String,

    pub name: String,
    pub uuid: Option<DomainXMLUuid>,
    pub genid: Option<uuid::Uuid>,
    pub title: Option<String>,
    pub description: Option<String>,
    pub os: OSXML,
    #[serde(default)]
    pub features: FeaturesXML,
    pub devices: DevicesXML,

    /// The maximum allocation of memory for the guest at boot time
    pub memory: DomainMemoryXML,

    pub on_poweroff: String,
    pub on_reboot: String,
    pub on_crash: String,
}

/// Domain state
#[derive(serde::Serialize, Debug, Copy, Clone)]
pub enum DomainState {
    NoState,
    Running,
    Blocked,
    Paused,
    Shutdown,
    Shutoff,
    Crashed,
    PowerManagementSuspended,
    Other,
}
